package com.ibm.able.examples.conversation;

//====================================================================
// Imports
//====================================================================

import javax.agent.Payload;
import org.jagent.acr.AcrPayload;

import com.ibm.able.Able;
import com.ibm.able.conversation.MessageForm;

import com.ibm.bilateral.OfferSet;

/**
 * This class represents the message-form used during the AMAB Conversation
 * by the seller to present an offer-set to the buyer
 * <p>
 * @see		com.ibm.able.conversation.MessageForm
 * @see		com.ibm.able.conversation.MessageFormFactory
 */

public class AmabOfferMessageForm implements MessageForm 
{
	// =====================================
	// Class variables.
	// =====================================
	protected static final String NAME_KEY = "message-name";
	protected static final String OBJECT_KEY = "offer-set";


	// =====================================
	// Instance variables
	// =====================================
	String name = "";	// A property. Must be non-null. Start with a safe default.
	OfferSet offerset = null;	// The constituent data. Must be set before calling formMessage().

	// =====================================
	// Constructors
	// =====================================
	/**
	 * Default Constructor
	 */
	public AmabOfferMessageForm() 
	{
		super();
	}

	/**
	 * Single Argument Constructor - as required by the MessageFormFactory
	 */
	public AmabOfferMessageForm(String newName) 
	{
		super();
		if (newName != null)
			name = newName;
	}
	

	// =====================================
	// Properties
	// =====================================
	/**
	 * Return the name
	 * <p>
	 * @return		the name 
	 */
	public String getName() 
	{
		return name;
	}


	/**
	 * Set the name of the Selection Message
	 * <p>
	 * @param		newName
	 * 				the name
	 */
	public void setName(String newName) 
	{
		if (newName == null)
			name = "";
		else
			name = newName;
	}


	// =====================================
	// Constituent data.
	// =====================================
	
	/**
	 * Return the offerset value
	 * <p>
	 * @return 	the offerset value
	 */
	public OfferSet getOfferSet() 
	{
		return offerset;
	}


	/**
	 * Set the offerset value
	 * <p>
	 * @param		selection
	 * 				the actual offerset value
	 */
	public void setOfferSet(OfferSet o) 
	{
		offerset = o;
	}


	/**
	 * Tries to set the selection
	 * <p>
	 * @param data Length-1 array containing an OfferSer as its first (and only) element.
	 */
	public void setData(Object[] data) throws IllegalArgumentException 
	{
		try 
		{
			OfferSet o = (OfferSet) data[0];
			offerset = o;
		}
		catch (Exception e) 
		{
			throw new IllegalArgumentException("Bad data[0]: expected String.");
		}
	}


	/**
	 * Returns the OfferSet, packed into a length-1 array. (To get the name, use getName()).
	 */
	public Object[] getData() 
	{
		return new Object[] { offerset };
	}


	/**
	 * Sets offerset to null. Does NOT change name.
	 */
	public void clearData() 
	{
		offerset = null;
	}


	// =====================================
	// MessageForm methods
	// =====================================
	/**
	 * internal function to create a payload for contituent data
	 */
	protected Payload makePayload(String localName, OfferSet o) 
	{
		AcrPayload payload = new AcrPayload();
		payload.set(NAME_KEY, localName);
		payload.set(OBJECT_KEY, o);
		return payload;
	}


	/**
	 * @see MessageForm#formMessagePayload()
	 */
	public Payload formMessagePayload() throws IllegalStateException 
	{
		if (offerset == null)
			throw new IllegalStateException("Message text is null.");

		Payload newPayload = this.makePayload(name, offerset);
		return newPayload;
	}


	/**
	 * @see MessageForm#sampleMessagePayload()
	 */
	public Payload sampleMessagePayload() 
	{
		Payload newPayload = this.makePayload(name,new OfferSet(10));
		return newPayload;
	}


	/**
	 * @see MessageForm#parseMessagePayload(Payload)
	 */
	public boolean parseMessagePayload(Payload messagePayload) 
	{
		
		String newName = null;
		OfferSet o = null;
		try 
		{
			newName = (String) messagePayload.get(NAME_KEY);
			o = (OfferSet) messagePayload.get(OBJECT_KEY);
		} 
		catch (Exception ex) 
		{
			Able.MessageLog.text(Able.MSG_WARNING,this,"parseMessagePayload",ex.getLocalizedMessage());
        	if (Able.TraceLog.isLogging()) 
        		Able.TraceLog.exception(Able.TRC_LOW,this,"parseMessagePayload",ex);
		}

		boolean isParsed = false;
		if (newName != null && o != null) 
			isParsed = newName.equals(name);

		if (isParsed) 
			offerset = o;

		return isParsed;
	}
}
