package com.ibm.able.examples.conversation;

//====================================================================
// Imports
//====================================================================

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.rmi.RemoteException;

import com.ibm.able.Able;
import com.ibm.able.AbleException;
import com.ibm.able.AbleObject;
import com.ibm.able.conversation.AbleJasConversationAgent;
import com.ibm.able.conversation.MessageFormFactory;
import com.ibm.bilateral.ClArgsImp;
import com.ibm.bilateral.Offer;
import com.ibm.bilateral.OfferSet;
import com.ibm.bilateral.RandomBN;
import com.ibm.bilateral.Response;
import com.ibm.bilateral.Seller;
import com.ibm.bilateral.TradingPartner;
import com.ibm.bilateral.able.ControlFile;

/**
 * This class is an example of an AbleBean that can be used in conjunction
 * with the ABLE Jas conversation agent to engage in an Asymmetric Multi-Attribute
 * Bilateral (AMAB) Negotitation conversation assuming the role of a seller.
 * <p>
 * This bean can be used inside the AbleJasConversationAgent to drive an AMAB conversation
 * as defined in the amab_cp.xml included in the examples/datafiles directory.
 * <p>
 * @see 	com.ibm.able.conversation.AbleJasConversationAgent
 */

public class AmabSellerLogic extends AbleObject implements Serializable
{
	/**
   	 * Serialized version identifier in form       YYYYMMDDVerRelModxx
   	 */
  	protected static final long serialVersionUID = 2002091900100300001L;
  	
	private final static boolean DEBUG = true;
	private Seller seller = null;
	TradingPartner partner;
	
	/**
	 * number of negotitations
	 */	
	private int numOfNegotiations = -1;
	
	/**
	 * number of rounds
	 */
	private int numOfRounds = -1;
	
	/**
	 * number of sellers
	 */
	private final int n_sellers = 1;
	
	/**
	 * strategy type
	 */
	private int strategyType = -1;;
	
	/**
	 * strategy parameters file
	 */
	transient private String strategyParamsFile = "";
	
	/**
	 * utility type
	 */
	private int utilityType = -1;
	
	/**
	 * utility parameters file
	 */
	private int utilityParamsType = -1;
	
	/**
	 * utility parameters file
	 */
	transient private String utilityParamsFile = "";
	

	/**
	 * returns the 	numbre of negotiations
	 * <p>
	 * @return		the number of negotiations
	 */
	public int getNumOfNegotiations()
	{
		return numOfNegotiations;
	}
	
	
	/**
	 * Sets the number of negotiations
	 * <p>
	 * @param		param
	 * 				the number of negotiations
	 */
	public void setNumOfNegotiations(int param)
	{
		numOfNegotiations = param;
	}
	
	
	/**
	 * Returns the number of rounds
	 * <p>
	 * @return		the number of rounds
	 */
	public int getNumOfRounds()
	{
		return numOfRounds;
	}
	
	
	/**
	 * Sets the number of rounds
	 * <p>
	 * @param		param
	 * 				the number of rounds
	 */
	public void setNumOfRounds(int param)
	{
		numOfRounds = param;
	}
	
	
	/**
	 * Returns the strategy type
	 * <p>
	 * @return		the strategy type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public int getStrategyType()
	{
		return strategyType;
	}
	

	/**
	 * Sets the strategy type
	 * <p>
	 * @param	param
	 * 			the strategy type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public void setStrategyType(int param)
	{
		strategyType = param;
	}
	

	/**
	 * Returns the strategy parameters file
	 * <p>
	 * @return		the strategy parameters file
	 */
	public String getStrategyParamsFile()
	{
		return strategyParamsFile;
	}
	

	/**
	 * Sets the strategy parameters file
	 * <p>
	 * @param		param
	 * 				the strategy parameters file
	 */
	public void setStrategyParamsFile(String param)
	{
		strategyParamsFile = param;
	}
	
	
	/**
	 * Returns the utility type
	 * <p>
	 * @return		the utility type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public int getUtilityType()
	{
		return utilityType;
	}
	
	
	/**
	 * Sets the utility type
	 * <p>
	 * @param	param
	 * 			the utility type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public void setUtilityType(int param)
	{
		utilityType = param;
	}
	
	
	/**
	 * Returns the utility parameters file
	 * <p>
	 * @param	param
	 * 			the utility type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public int getUtilityParamsType()
	{
		return utilityParamsType;
	}
	
	
	/**
	 * Sets the utility parameter type
	 * <p>
	 * @param		param
	 * 				the utility parameter type
	 * <p>
	 * @see		com.ibm.able.examples.conversation.AmabParams
	 */
	public void setUtilityParamsType(int param)
	{
		utilityParamsType = param;
	}
	

	/**
	 * Returns the utility parameters file
	 * <p>
	 * @return		the utility parameter file
	 */
	public String getUtilityParamsFile()
	{
		return utilityParamsFile;
	}

	
	/**
	 * Sets the utility parameters file
	 * <p>
	 * @param		file
	 * 				the utility parameter file
	 */
	public void setUtilityParamsFile(String param)
	{
		utilityParamsFile = param;
	}
	
	
	/**
	 * custom writeObject to handle storing of file name paths
	 */
	private void writeObject(ObjectOutputStream out) throws IOException
	{
		out.defaultWriteObject();
		out.writeObject(AbleJasConversationAgent.getSerializableFileName(strategyParamsFile));
		out.writeObject(AbleJasConversationAgent.getSerializableFileName(utilityParamsFile));
	}


	/**
	 * custom readObject to handle restoring of file name paths
	 * and initializing message forms
	 */
	private void readObject(ObjectInputStream in) throws IOException,ClassNotFoundException
	{
		in.defaultReadObject();
		String temp = (String)in.readObject();
		strategyParamsFile = AbleJasConversationAgent.getActualFileName(temp);
		
		temp = (String)in.readObject();
		utilityParamsFile = AbleJasConversationAgent.getActualFileName(temp);
		
		Class c1 = null, c2 = null;
		c1 = com.ibm.able.examples.conversation.AmabSelectionMessageForm.class;
		c2 = com.ibm.able.examples.conversation.AmabOfferMessageForm.class;
			
		MessageFormFactory.addLocalTableEntry("AmabSelectionMessageForm",c1);
		MessageFormFactory.addLocalTableEntry("AmabOfferMessageForm",c2);
	}
	
	
	/**
	 * Constructor for AmabSellerLogic
	 */
	public AmabSellerLogic() throws RemoteException 
	{
		super();
	}

	/**
	 * Constructor for AmabSellerLogic
	 */
	public AmabSellerLogic(String theName) throws RemoteException 
	{
		super(theName);
	}

	/**
	 * Constructor for AmabSellerLogic
	 */
	public AmabSellerLogic(String theName, String theComment)
		throws RemoteException 
	{
		super(theName, theComment);
	}


	/**
	 * internal function to check the validity of internal variables
	 */
	private boolean validParams()
	{
		if(	(numOfNegotiations > 0) &&
			(numOfRounds > 0) &&
			(strategyType >= 0) &&
			!(strategyParamsFile.equals("")) &&
			(utilityType >= 0) &&
			(utilityParamsType >= 0) &&
			!(utilityParamsFile.equals(""))
		  )
		  	return true;
		else
			return false;
	}
	
	
	/**
	 * @see	com.ibm.able.AbleObject#init()
	 */
	public void init() throws AbleException 
  	{
  		// check paramter validity
  		if(!validParams())
  			return;
  			
  		super.init();
  		
  		// initialize the actual seller logic
		ClArgsImp cl = new ClArgsImp();
		cl.setNumberOfNegotiations(numOfNegotiations);
		cl.setNumberOfRounds(numOfRounds);
		
	 	RandomBN.setSeed(cl.getRandomSeed());
	  	
	 	ControlFile cf = new ControlFile();
	  	Seller[] sellers = null;
	  	try
	  	{
	  		cf.initSellers(n_sellers,strategyType,strategyParamsFile,utilityType,utilityParamsType,utilityParamsFile);
			sellers = cf.createSellers(n_sellers, 0, cl);
	  	}
	  	catch(Exception ex)
	  	{
	  		Able.MessageLog.text(Able.MSG_WARNING,this,"init",ex.getLocalizedMessage());
        	if (Able.TraceLog.isLogging()) 
        		Able.TraceLog.exception(Able.TRC_LOW,this,"init",ex);
	  	}
		
		int s = RandomBN.irand(n_sellers);
		seller = sellers[s];
		seller.sellerInitNegotiation(partner);
		
		Class c1 = null, c2 = null;
		c1 = com.ibm.able.examples.conversation.AmabSelectionMessageForm.class;
		c2 = com.ibm.able.examples.conversation.AmabOfferMessageForm.class;
			
		MessageFormFactory.addLocalTableEntry("AmabSelectionMessageForm",c1);
		MessageFormFactory.addLocalTableEntry("AmabOfferMessageForm",c2);
  	}
  	
 
 	/**
	 * Internal function to handle the decision making process - decision point names
	 * as defined in examples/datafiles/amab_cp.xml
	 */
  	private OfferSet decide(String whichDecision, Response rsp) 
	{
		OfferSet os = null;
		if (whichDecision.equals("Start")) 
		{
			boolean enterNego = true;
			if (!enterNego)
			{
				os = OfferSet.createQuitNoOffers();
			}
			else
     			os = seller.sellerMakeOffer();
		}
		else if (whichDecision.equals("InitialOfferRefused")) 
		{
	        seller.sellerReceiveResponse(rsp);
	        os = seller.sellerMakeOffer();
	        
		} 
		else if (whichDecision.equals("InitialOfferAccepted")) 
		{
	        seller.sellerReceiveResponse(rsp);
	        os = seller.sellerMakeOffer();
		}
		else if (whichDecision.equals("ImprovedOfferProcessed")) 
		{
	        seller.sellerReceiveResponse(rsp);
	        os = seller.sellerMakeOffer();
		} 
		else if (whichDecision.equals("NegotiationOver")) 
		{
			Offer fo = seller.sellerConfirm();
			os = new OfferSet(1);
			os.addOffer(fo);
		} 
		else if (whichDecision.equals("DealConfirmed")) 
		{
			// Deal is closed successfully
			seller.sellerCompleteNegotiation(true);
			partner = null;
			seller.sellerInitNegotiation(partner);
		}
		else if (whichDecision.equals("NoDeal")) 
		{
			// Deal FAILS
			seller.sellerCompleteNegotiation(false);
			partner = null;
			seller.sellerInitNegotiation(partner);
		}
		else
			throw new IllegalArgumentException("unknown decision: " + whichDecision);
			
		if (DEBUG)
			System.out.println("SellerLogic " + whichDecision + " decision handler");
			
			// JEH 10/12/02
			if (trace.isLogging())
				trace.text(Able.TRC_LOW,this,"decide","Decision point = " + whichDecision);
	

		return os;
	}

	
	/**
	 * Decision logic connection function for the AmabSellerLogic bean
	 * <p>
	 * @param		conversationId
	 * 				the conversation identifier
	 * <p>
	 * @param		decisionPoint
	 * 				the decicion point (state name) at which a deicision is to be made
	 * <p>
	 * @param		decisionData
	 * 				object array representing the data used in making a decision
	 * <p>
	 * @return		an object array, with the first element of the array representing
	 * 				the decision made and the second parameter itself an object array
	 * 				that is to be used to construct the message form for an outgoing message
	 */
  	public Object[] amabSellerLogic(String conversationId, String decisionPoint, Object decisionPointData[])
  	{
  		Response response = null;
		
		try
		{
			Integer i = (Integer)decisionPointData[0];
			response = new Response(i.intValue());
		}
		catch(Exception e)
		{
			// Start/Quit/NoSelection Response 
			if(decisionPointData.length > 0)
			{
				String s = (String)decisionPointData[0];
			
				if(s.equals("Quit"))
				{
					response = Response.QUIT_RESPONSE;
					//response = new Response(true);
				}
			
				if(s.equals("NoSelection"))
					response = new Response();
			}
			else
			{
				// start state - dont do anything
			}
		}
		
		OfferSet os = decide(decisionPoint,response);
		String decision = null;
		Object[] decisionData = null;

		if (os != null) 
		{	
			if(os.isQuit())
			{
				decision = new String("Quit");
				decisionData = new Object[] { "Quit" };
			}
			else
			{
				decision = new String("Offer");
				decisionData = new Object[] { os };
			}
			
			Object returnData[] = new Object[2];
  		
	  		returnData[0] = decision;
			returnData[1] = decisionData;
  				
  			return returnData;	
		}
		else
			return null;
  	}
}
