package com.ibm.able.examples.conversation;

//====================================================================
// Imports
//====================================================================

import java.rmi.RemoteException;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import com.ibm.able.Able;
import com.ibm.able.AbleObjectCustomizer;


/**
 * This class provides the customizer for the AmabSellerLogic bean. Adds
 * an AmabSellerLogicPanel to the customizer panels provided by 
 * AbleObjectCustomizer
 * <p>
 * @see com.ibm.able.AbleObjectCustomizer
 * @see com.ibm.able.examples.conversation.AmabSellerLogicPanel
 *
 */
public class AmabSellerLogicCustomizer extends AbleObjectCustomizer 
{
	//-----------------------------------------------------------------
    // Class variables
    //-----------------------------------------------------------------   


    //-----------------------------------------------------------------
    // Member variables
    //-----------------------------------------------------------------
    private AmabSellerLogicPanel myPanel = null;
    
    //-----------------------------------------------------------------
    // Constructors
    //-----------------------------------------------------------------
    /**
     *  Construct a Amab Seller Logic Customizer dialog.  Dialog is
     *  not modal so other windows can be active at the same time.
     *
     * @param theFrame The frame from which the dialog is displayed.
     * @param theTitle The default name to display in the dialog's title bar.
     *              This will be replaced with the object name when the object is set.
     */
    public AmabSellerLogicCustomizer(JFrame theFrame, String theTitle) 
    {
        super(theFrame, theTitle, false);   // non-modal
     
        // pass this customizer to the base customizer so OK calls this.updateObject
        setCustomizer(this);          
     
        // create the panel;  pass this customizer so it can call this.updateObject too
        // and so it can register property listeners with the base customizer
        myPanel = new AmabSellerLogicPanel(this);
        insertTab("Seller",myPanel,1);
    }
    

    /**
     *  Construct a Amab Seller Logic Customizer dialog.  Dialog is
     *  not modal so other windows can be active at the same time.
     *
     */
    public AmabSellerLogicCustomizer() 
    {
        this(new JFrame(), "AMAB Seller Logic Customizer") ;
    }


    //--------------------------------------------------------------------
    // Methods inherited from Customizer interface
    //--------------------------------------------------------------------
    /**
     * Write the data from the object to the panel(s) provided
     * by this customizer. The panel saves a reference to the
     * bean being customized.
     * Pass the object and its data to any inherited panel.
     *
     * @param     theBean
     *            The bean that is to be changed.
     * @see com.ibm.able.AbleObjectCustomizer
     */
    public void setObject(Object theBean) 
    {
        super.setObject(theBean);
   		try 
	    {
	        myPanel.getDataFromBean((AmabSellerLogic)theBean);
	    }
	    catch (RemoteException e) 
	    {
            JOptionPane.showMessageDialog(
               this,                                // Parent
               e.toString(),                        // Msg
               "AmabSellerLogicCustomizer Error: setObject()",         // Title
               JOptionPane.ERROR_MESSAGE            // Severity
            );
	    }
    }


    /**
     * Update the bean that is being customized.  This method is 
     * called when the dialog's OK button is pressed.
     *
     * @see com.ibm.able.AbleObjectCustomizer
     * @exception RemoteException is thrown when the panel values
     *            cannot be set on the bean.
     */
    public void updateObject() throws RemoteException 
    {
        super.updateObject();
	    myPanel.setDataOnBean();
    }


    /**
     * Determine the copyright of this class.
     *
     * @return    A String containing this class's copyright statement.
     *            <p>
     */
    public static String Copyright() 
    {
      return Able.Copyright;
    }
}
