package com.ibm.able.examples.conversation;

//====================================================================
// Imports
//====================================================================
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.ibm.able.Able;
import com.ibm.able.AbleCustomizerPanel;
import com.ibm.able.AbleHelp;
import com.ibm.able.AbleObjectCustomizer;
import com.ibm.able.AblePanelHelper;

/**
 * This class provides the graphical editor panel for the 
 * AmabSellerLogic to specify various parameters required to create an AMAB Seller Object
 * It provides a tab used by the AmabSellerLogicCustomizer and any customizers that extend it.
 * <p>
 * @see AmabSellerLogicCustomizer
 */

public class AmabSellerLogicPanel extends JPanel implements AbleCustomizerPanel 
{
    //-----------------------------------------------------------------
    // Class variables
    //-----------------------------------------------------------------

    private final static String NoneSelected = "<none>";

    //-----------------------------------------------------------------
    // Member variables
    //-----------------------------------------------------------------
	private int numOfNegotiations = -1;
	private int numOfRounds = -1;
	private int strategyType = -1;;
	private String strategyParamsFile = "";
	private int utilityType = -1;
	private int utilityParamsType = -1;
	private String utilityParamsFile = "";
	     
    private AmabSellerLogic seller = null;
    private AmabSellerLogicCustomizer myCustomizer = null;
    
    private JTextField myNegotiationsTextField = null;
    private JTextField myRoundsTextField = null;
        
    private JComboBox myStrategyComboBox = null;    
    private JTextField myStrategyFileTextField = null;    

    private JComboBox myUtilTypeComboBox = null;    
    private JComboBox myUtilParamsTypeComboBox = null;        
    private JTextField myUtilParamsFileTextField = null;    

    private JButton mySetValuesButton = null;
    private boolean myListenersEnabled = false;
    
    //-----------------------------------------------------------------
    // Constructors
    //-----------------------------------------------------------------

    /**
     * No default constructor.
     */
    private AmabSellerLogicPanel() 
    {}


    /**
     * Construct the panel for the customizer.
     * @param theCustomizer provides a reference to the customizer so 
     *                      PropertyChangeEvents can be registered.
     */
    public AmabSellerLogicPanel(AmabSellerLogicCustomizer theCustomizer) 
    {
        super();
        myCustomizer = theCustomizer;
        this.setLayout(null);
        this.setBounds(5, 25, AbleObjectCustomizer.TabWidth, AbleObjectCustomizer.TabHeight);
        this.setPreferredSize(new Dimension(AbleObjectCustomizer.TabWidth, AbleObjectCustomizer.TabHeight));
        initializePanel();
    }


    //-----------------------------------------------------------------
    // Member methods
    //-----------------------------------------------------------------
    /**
     * Save a reference to the agent that is being customized.
     * Get the data from the bean and set the panel components.
     *
     * @param     theObject
     *            The AbleObject that is to be changed.
     * @exception RemoteException is thrown when data cannot be retrieved
     *            from a remote object.
     */
	public void getDataFromBean (Object theObject) throws RemoteException 
	{	
	    if (seller == null) 
    	{
            seller = (AmabSellerLogic)theObject;
        }
	    
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        setListenersEnabled(false);
        try 
        {
		    myStrategyComboBox.removeAllItems();
		    Vector v = new Vector();
		    v.insertElementAt(this.NoneSelected, 0);
	    	v.insertElementAt("ZI",1);
 	        myStrategyComboBox.setModel(new DefaultComboBoxModel(v));
        
	   		myUtilTypeComboBox.removeAllItems();
	   		v = new Vector();
		    v.insertElementAt(this.NoneSelected, 0);
	    	v.insertElementAt("CES",1);
 	        myUtilTypeComboBox.setModel(new DefaultComboBoxModel(v));

    		myUtilParamsTypeComboBox.removeAllItems();
            v = new Vector();
		    v.insertElementAt(this.NoneSelected, 0);
	    	v.insertElementAt("FIXED",1);
	    	v.insertElementAt("RANDOM",2);	    	
 	        myUtilParamsTypeComboBox.setModel(new DefaultComboBoxModel(v));
 	        
            numOfNegotiations = seller.getNumOfNegotiations();
            if(numOfNegotiations > 0)
            	myNegotiationsTextField.setText(Integer.toString(numOfNegotiations));
            	
			numOfRounds = seller.getNumOfRounds();
            if(numOfRounds > 0)
            	myRoundsTextField.setText(Integer.toString(numOfRounds));
			
			strategyType = seller.getStrategyType();
			if(strategyType >= 0)
			{
				if(strategyType == AmabParams.ZI_STRATEGY)
					myStrategyComboBox.setSelectedItem("ZI");
			}
			else
				myStrategyComboBox.setSelectedIndex(0);
			
			strategyParamsFile = seller.getStrategyParamsFile();
			myStrategyFileTextField.setText(strategyParamsFile);
			
			utilityType = seller.getUtilityType();
			if(utilityType >= 0)
			{
				if(utilityType == AmabParams.CES_UTILITY)
					myUtilTypeComboBox.setSelectedItem("CES");
			}
			
			utilityParamsType = seller.getUtilityParamsType();
			if(utilityParamsType >= 0)
			{
				if(utilityParamsType == AmabParams.FIXED_UTILITY_PARAMS)
					myUtilParamsTypeComboBox.setSelectedItem("FIXED");
				else if(utilityParamsType == AmabParams.RANDOM_UTILITY_PARAMS)
					myUtilParamsTypeComboBox.setSelectedItem("RANDOM");
			}
			
			utilityParamsFile = seller.getUtilityParamsFile();
			myUtilParamsFileTextField.setText(utilityParamsFile);
	    } 
        finally 
        {
            setListenersEnabled(true);
            setButtonStates();
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
	}

	
    /**
     * Update the object that is being customized.
     *
     * @exception RemoteException is thrown when data cannot be set
     *            on a remote object.
     */
	public void setDataOnBean() throws RemoteException 
	{
		seller.setNumOfNegotiations(numOfNegotiations);
		seller.setNumOfRounds(numOfRounds);
		seller.setStrategyType(strategyType);
		seller.setStrategyParamsFile(strategyParamsFile);
		seller.setUtilityType(utilityType);
		seller.setUtilityParamsType(utilityParamsType);
		seller.setUtilityParamsFile(utilityParamsFile);		
	}
    

    /**
     * Initialize the panel.
     *
     */
    private void initializePanel() 
    {
    	int y = 10;
        int h = AbleObjectCustomizer.TabHeight / 10;  // 8 rows  // was 35 pels

		AblePanelHelper.addLabel(this, 20, 7, 150, "NEGOTIATION");
        AblePanelHelper.addLabel(this, 20, y+=1.25*h, 130, "No. Of Negotiations:");
        myNegotiationsTextField = AblePanelHelper.addTextField(this, 160, y, 30);
		myNegotiationsTextField.getDocument().addDocumentListener(new TextListener(TextListener.NEGOTIATION));         
        myNegotiationsTextField.addActionListener(new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                negotiationTextField_actionPerformed();
            }
        });
        AblePanelHelper.addLabel(this, 250, y, 100, "No. Of Rounds:");
        myRoundsTextField = AblePanelHelper.addTextField(this, 360, y, 30);
		myRoundsTextField.getDocument().addDocumentListener(new TextListener(TextListener.ROUNDS));        
        myRoundsTextField.addActionListener(new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                roundsTextField_actionPerformed();
            }
        });
        
        
        AblePanelHelper.addLabel(this, 20, y+=1.5*h, 150, "STRATEGY");
        AblePanelHelper.addLabel(this, 20, y+=1.25*h, 50, "Strategy:");        
        myStrategyComboBox = AblePanelHelper.addComboBox(this, 90, y, 80);
        myStrategyComboBox.addItemListener(new ItemListener() 
        {
            public void itemStateChanged(ItemEvent e) 
            {
                strategyComboBox_itemStateChanged(e);
            }
        });
        AblePanelHelper.addLabel(this, 200, y, 90, "Params. File:"); 
        myStrategyFileTextField = AblePanelHelper.addTextField(this, 300, y, 100);
		myStrategyFileTextField.getDocument().addDocumentListener(new TextListener(TextListener.STRATEGY_FILE));     
        myStrategyFileTextField.addActionListener(new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                strategyFileTextField_actionPerformed();
            }
        });
        AblePanelHelper.addButton(this, 410, y, 80, "Browse",
                (new ActionListener() 
                {
                	public void actionPerformed(ActionEvent e) 
                	{
                    	browseButtonClicked("in",myStrategyFileTextField,e);
                	}
                }),
          		"Select a file for strategy params");
        
        AblePanelHelper.addLabel(this, 20, y+=1.5*h, 150, "UTILITY");
        AblePanelHelper.addLabel(this, 20, y+=1.25*h, 50, "Utility:");        
        myUtilTypeComboBox = AblePanelHelper.addComboBox(this, 90, y, 80);
        myUtilTypeComboBox.addItemListener(new ItemListener() 
        {
            public void itemStateChanged(ItemEvent e) 
            {
                utilTypeComboBox_itemStateChanged(e);
            }
        });
        
        AblePanelHelper.addLabel(this, 20, y+=0.8*h, 100, "Utility Params:");        
        myUtilParamsTypeComboBox = AblePanelHelper.addComboBox(this, 130, y, 80);
        myUtilParamsTypeComboBox.addItemListener(new ItemListener() 
        {
            public void itemStateChanged(ItemEvent e) 
            {
                utilParamsComboBox_itemStateChanged(e);
            }
        });
        AblePanelHelper.addLabel(this, 240, y, 80, "Params. File:"); 
        myUtilParamsFileTextField = AblePanelHelper.addTextField(this, 330, y, 70);
		myUtilParamsFileTextField.getDocument().addDocumentListener(new TextListener(TextListener.UTIL_PARAMS_FILE));        
        myUtilParamsFileTextField.addActionListener(new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                utilParamsFileTextField_actionPerformed();
            }
        });
        AblePanelHelper.addButton(this, 410, y, 80, "Browse",
                (new ActionListener() 
                {
                	public void actionPerformed(ActionEvent e) 
                	{
                    	browseButtonClicked("in",myUtilParamsFileTextField,e);
                	}
                }),
          		"Select a file for strategy params");
          		        
        mySetValuesButton = AblePanelHelper.addButton(this, 250, y+=h, 150, "Set Values",
                					(new ActionListener() 
                						 {
                							public void actionPerformed(ActionEvent e) 
                							{
                    							setValuesButtonClicked();
                							}
                						}),
						       		"Begin a new conversation");
		mySetValuesButton.setEnabled(false);  
	    myStrategyFileTextField.setEnabled(false);    
    	myUtilParamsFileTextField.setEnabled(false);    	    
    }

    
    /**
     *  When a property change event occurs, the customizer will call this method.
     *  The panel is refreshed from the AbleObject reference.
     */
     public void refreshPanel(PropertyChangeEvent theEvent) throws RemoteException 
     {
        setListenersEnabled(false);
        getDataFromBean(seller);
        setButtonStates();
        setListenersEnabled(true);
     }


    //-----------------------------------------------------------------
    // ActionEvent methods
    //-----------------------------------------------------------------
    /**
     * React to browse button bein clicked
     */
	private void browseButtonClicked(String extension,JTextField textField,ActionEvent e) 
    {
    	String newDir = null;
    	
       	String[] extensions = {extension};
       	newDir = AblePanelHelper.setTextFieldFromFileDialog(textField, "(*." + extension + ")", extensions, myCustomizer.getWorkingDirectory());

        if (newDir != null) 
        {
            myCustomizer.setWorkingDirectory(newDir);
        }
    }
    
    
    /**
     *  Set the buyer paramters in the AmabBuyerLogic Bean
     */
    private void setValuesButtonClicked() 
    {
    	try
    	{
     	   setDataOnBean();
    	}
    	catch(Exception e)
    	{
    		JOptionPane.showMessageDialog(
                    myCustomizer,           	// Parent
                    e.toString(),               // Msg
                    "AmabSellerLogicPanel Warning: setValuesButtonClicked()",     // Title
                    JOptionPane.WARNING_MESSAGE          // Severity
                );
    	}
   	}


	/**
	 * react to change in the negotiation text field
	 */
	private void negotiationTextField_actionPerformed()
	{
       if (!myListenersEnabled) 
        	return;

		String text = myNegotiationsTextField.getText().trim();
		if(text.length() > 0)
		{
			try
			{
				int temp = Integer.parseInt(text);
				numOfNegotiations = temp;
			}
			catch(Exception exception)
			{
				// ignore
				numOfNegotiations = -1;
			}
		}
		else
			numOfNegotiations = -1;

        setButtonStates() ;
	}


	/**
	 * react to change in the rounds text field
	 */
	private void roundsTextField_actionPerformed()
	{
       if (!myListenersEnabled) 
        	return;

		String text = myRoundsTextField.getText().trim();
		if(text.length() > 0)
		{
			try
			{
				int temp = Integer.parseInt(text);
				numOfRounds = temp;
			}
			catch(Exception exception)
			{
				// ignore
				numOfRounds = -1;
			}
		}
		else
			numOfRounds = -1;

        setButtonStates() ;
	}


	/**
	 * react to change in the strategy file text field
	 */
	private void strategyFileTextField_actionPerformed()
	{
       if (!myListenersEnabled) 
        	return;

		String text = myStrategyFileTextField.getText().trim();
		if(text.length() > 0)
		{
			strategyParamsFile = text;
		}
		else
			strategyParamsFile = "";

        setButtonStates() ;
	}
	
	
	/**
	 * react to change in the utility paramters file text field
	 */
	private void utilParamsFileTextField_actionPerformed()
	{
       if (!myListenersEnabled) 
        	return;

		String text = myUtilParamsFileTextField.getText().trim();
		if(text.length() > 0)
		{
			utilityParamsFile = text;
		}
		else
			utilityParamsFile = "";

        setButtonStates() ;
	}
	
    
   	/**
     *  React to a selection change in the strategy combobox.
     */
    private void strategyComboBox_itemStateChanged(ItemEvent e) 
    {
        if (!myListenersEnabled) 
        	return;
        
        if(myStrategyComboBox.getSelectedIndex() > 0)
        {
        	String selection = (String)myStrategyComboBox.getSelectedItem();
        	if(selection.equals("ZI"))
        		strategyType = AmabParams.ZI_STRATEGY;
        }
        else
        	strategyType = -1;
        
        setButtonStates() ;
    }


	/**
     *  React to a selection change in the utility type combobox.
     */
	private void utilTypeComboBox_itemStateChanged(ItemEvent e) 
    {
        if (!myListenersEnabled) 
        	return;
        
        if(myUtilTypeComboBox.getSelectedIndex() > 0)
        {
        	String selection = (String)myUtilTypeComboBox.getSelectedItem();
        	if(selection.equals("CES"))
        		utilityType = AmabParams.CES_UTILITY;
        }
        else
        	utilityType = -1;
        
        setButtonStates() ;
    }
    
    
    /**
     *  React to a selection change in the utility parameters combobox.
     */
   	private void utilParamsComboBox_itemStateChanged(ItemEvent e) 
    {
        if (!myListenersEnabled) 
        	return;
        
        if(myUtilParamsTypeComboBox.getSelectedIndex() > 0)
        {
        	String selection = (String)myUtilParamsTypeComboBox.getSelectedItem();
        	if(selection.equals("RANDOM"))
        		utilityParamsType = AmabParams.RANDOM_UTILITY_PARAMS;
        	if(selection.equals("FIXED"))
        		utilityParamsType = AmabParams.FIXED_UTILITY_PARAMS;
        }
        else
        	utilityParamsType = -1;
        
        setButtonStates() ;
    }


    //-----------------------------------------------------------------
    // Utility methods
    //-----------------------------------------------------------------
    /**
     *  Set the button states based on the selected fields.
     */
    private void setButtonStates() 
    {
    	if(	(numOfNegotiations > 0) &&
			(numOfRounds > 0) &&
			(strategyType >= 0) &&
			!(strategyParamsFile.equals("")) &&
			(utilityType >= 0) &&
			(utilityParamsType >= 0) &&
			!(utilityParamsFile.equals(""))
		  )
		  	mySetValuesButton.setEnabled(true);
		else
		  	mySetValuesButton.setEnabled(false);
    }

    
    
    /**
     *  Turn off the actions for the listeners to prevent unwanted updates.
     */
    private void setListenersEnabled(boolean theFlag) 
    {
        myListenersEnabled = theFlag;
        
        myNegotiationsTextField.setEnabled(theFlag);
    	myRoundsTextField.setEnabled(theFlag);
      	myStrategyComboBox.setEnabled(theFlag);    
		myUtilTypeComboBox.setEnabled(theFlag);    
    	myUtilParamsTypeComboBox.setEnabled(theFlag);        
    }
        

    //=================================================================
    //
    // Inner classes
    //
    //=================================================================
	/**
     * This class changes the button states when a text field changes.
     */
    class TextListener implements DocumentListener
    {
    	 public static final int NEGOTIATION = 1;
    	 public static final int ROUNDS = 2;
    	 public static final int STRATEGY_FILE = 3;
    	 public static final int UTIL_PARAMS_FILE = 4;
    	 int field = -1;
    	 
    	 public TextListener(int field)
    	 {
    	 	this.field = field;
    	 }
    	 
         public void changedUpdate(DocumentEvent e) 
         {
             if (!myListenersEnabled) return;
             
             switch(field)
             {
             	case TextListener.NEGOTIATION:
             			negotiationTextField_actionPerformed();
             			break;
             	
             	case TextListener.ROUNDS:
             			roundsTextField_actionPerformed();
             			break;
             			
             	case TextListener.STRATEGY_FILE:
             			strategyFileTextField_actionPerformed();
             			break;
             			
             	case TextListener.UTIL_PARAMS_FILE:
             			utilParamsFileTextField_actionPerformed();
             			break;
             			
             	default:
             }
             
             setButtonStates() ;
         }
      
      
         public void insertUpdate(DocumentEvent e) 
         {
             changedUpdate(e);
         }
      
         public void removeUpdate(DocumentEvent e) 
         {
      		 changedUpdate(e);       
         }
    }  

    //------------------------------------------------------------------
    // Required by AbleCustomizerPanel interface
    //------------------------------------------------------------------

    /**
     * Get the html file name to be displayed.  
     *
     * @return A String containing the html file to display in the browser.
     */
     public String getHelpFileName() 
     { 
     	return AbleHelp.helpFilesPath + 
               "doc-files/AbleUserDefinedFunctionHelp.html" ; 
	 }


    /** Get the URL file context to be displayed. Able.HomeUrl is the
     * Able context.
     *
     * @see Able
     *
     * @return A URL containing the html file to display in the browser.
     */
     public URL getHelpContext() { return Able.HomeUrl; } 

    /**
     * Determine the copyright of this class.
     *
     * @return    A String containing this class's copyright statement.
     *            <p>
     */
    public static String Copyright() 
    {
      return Able.Copyright;
    }

}
    
