package com.ibm.able.examples.rules;

//====================================================================
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// Product: com.ibm.able.examples.rules
//
// (C) Copyright IBM Corp. 2001 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
//
//                             DISCLAIMER
//                             ----------
//
// This material contains programming source code for your consideration.
// These examples have not been thoroughly tested under all conditions.
// IBM, therefore, cannot guarantee or imply reliability, serviceability,
// performance or function of these programs.  All programs contained
// herein are provided to you "AS IS".  THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY
// DISCLAIMED.
//
//====================================================================

//====================================================================
// Imports
//====================================================================
import java.io.Serializable;
import java.util.Vector;

import com.ibm.able.Able;

/**
 * A grocery bag has an id, and contains items sized small, medium, or large.
 * @version  $Revision: 1.6 $, $Date: 2001/09/27 20:59:15 $
 */
public class GroceryBag implements Serializable {

   /**
    * Serialized version identifier in form       YYYYMMDDVerRelModxx
    */
   static final long serialVersionUID = 2001100500100300000L;
   private static int nextId = 0;
   private Vector contents = new Vector();
   // protected so ruleset can use object.member notation instead of bulkier object.getMember()
   protected int id = -1;
   protected int numSmallItems = 0;
   protected int numMediumItems = 0;
   protected int numLargeItems = 0;
   private boolean full = false;

   public GroceryBag() {
      id = nextId++;
   }

   public void addItem(Grocery item) {
      contents.add(item); // add the item to this bag
      item.setBag(id); // show item has been bagged and is now unavailable
      if (item.getSize().equals("large"))
         numLargeItems++;
      else if (item.getSize().equals("medium"))
         numMediumItems++;
      else if (item.getSize().equals("small"))
         numSmallItems++;
      setFull();
   }

   private void setFull() {
      if (numLargeItems >= 6) {
         full = true;
      } else {
         full = false;
      }
   }

   public void removeItem(Grocery item) {
      if (contents.contains(item)) {
         contents.remove(item);
         if (item.getSize().equals("large"))
            numLargeItems--;
         else if (item.getSize().equals("medium"))
            numMediumItems--;
         else if (item.getSize().equals("small"))
            numSmallItems--;
         setFull();
      }
   }

   public Vector getContents() {
      return contents;
   }

   public int getNumLargeItems() {
      return numLargeItems;
   }

   public int getNumMediumItems() {
      return numMediumItems;
   }

   public int getNumItems() {
      return contents.size();
   }

   public int getId() {
      return id;
   }

   public boolean isFull() {
      return full;
   }
   public static void reset() {
      nextId = 0;
   }

   public String toString() {
      return "bag=(id=" + id + " contents=" + contents + " full=" + full + ")";
   } 
   
   /**
           * Determine the copyright of this class.
           *
           * @return    A String containing this class's copyright statement.
           *            <p>
           */
   public static String Copyright() {
      return Able.Copyright;
   }
}
